import { DirectUpload } from "@rails/activestorage";
import Type from "../utils/type";
import mimeType from "../utils/mime_type";

const MB_SIZE = 1000;

export default class Uploader {
  constructor(file, url, progress) {
    this.file = file;
    this.url = url;
    this.progress = progress;
    this.upload = new DirectUpload(this.file, this.url, this);
  }

  start(callback) {
    this.progress.value = 0;
    // this.dispatch("start")
    this.upload.create((error, blob) => callback(error, blob));
  }

  // dispatchEvent(element, type, eventInit = {}) {
  //   const { disabled } = element
  //   const { bubbles, cancelable, detail } = eventInit
  //   const event = document.createEvent("Event")

  //   event.initEvent(type, bubbles || true, cancelable || true)
  //   event.detail = detail || {}

  //   try {
  //     element.disabled = false
  //     element.dispatchEvent(event)
  //   } finally {
  //     element.disabled = disabled
  //   }

  //   return event
  // }

  // dispatch(name, detail = {}) {
  //   detail.file = this.file
  //   return this.dispatchEvent(this.input, `direct-upload:${name}`, { detail })
  // }

  // Called by DirectUpload
  directUploadWillStoreFileWithXHR(request) {
    request.upload.addEventListener("progress", event =>
      this.directUploadDidProgress(event)
    );
  }

  directUploadDidProgress(event) {
    // const progress = event.loaded / event.total * 100
    // if (progress)
    //   this.dispatch("progress", { progress })
    this.progress.value = (event.loaded / event.total) * 100;
  }

  validateFile(spec) {
    return this.validateFileAccordingToRules(spec.accepts);
  }

  validateFileAccordingToRules(spec) {
    if (Type.isObject(spec)) {
      let validFileType = false;
      [spec.fileType].flat().forEach(key => {
        const acceptsRegex = new RegExp(mimeType[key]);
        if (acceptsRegex.test(this.file.type)) {
          validFileType = true;
        }
      });

      if (!validFileType) {
        alert(spec.fileTypeErrorText || "Invalid file type");
        return false;
      }

      if (spec.maxFileSize) {
        if (this.file.size > spec.maxFileSize * MB_SIZE) {
          alert(spec.maxFileSizeErrorText || "File too big");
          return false;
        }
      }
    }

    return true;
  }
}
